/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : deep_standby_cancel.c
* $Rev: 1258 $
* $Date:: 2014-09-05 11:38:03 +0900#$
* Description : Canceling from deep standby mode sample
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "r_typedefs.h"
#include "devdrv_rtc.h"         /* RTC Driver Header */
#include "iodefine.h"
#include "rza_io_regrw.h"
#include "deep_standby_cancel.h"

#ifdef __CC_ARM
#pragma arm section code   = "CODE_RESET"
#pragma arm section rodata = "CONST_RESET"
#pragma arm section rwdata = "DATA_RESET"
#pragma arm section zidata = "BSS_RESET"
#endif

/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/


/******************************************************************************
Private global variables and functions
******************************************************************************/
/* Time when deep standby mode is cancelled */
static rtc_time_t stb_rtc_time;

/* Alarm interrupt generation of real time clock during deep standby mode */
/* (STB_GENERATE_ALARM_INT    : Alarm interrupt generated,                */
/*  STB_NO_GENERATE_ALARM_INT : Alarm in terrupt NOT generated)           */
static uint32_t   stb_rtc_state;


/*******************************************************************************
* Function Name: STB_CancelDeepStandby
* Description  : Determine if this function has been called after returning from 
*              : deep standby mode with reference to the cancel source confirmation 
*              : flag for the deep standby cancel source flag register (DSFR). 
*              : If this function has been called after returning from deep standby 
*              : mode, the processing corresponding to the deep standby mode cancel 
*              : source should be executed to release the retention of pin state.
*              : In the sample code, if the deep standby mode cancel source is the 
*              : alarm interrupt of the RTC, the RTCARF bit in the deep standby 
*              : cancel source flag register (DSFR) is cleared to "0", and each 
*              : counter value of the RTC is stored in the area retained in the 
*              : large-capacity on-chip RAM as the time when the deep standby mode 
*              : is cancelled. The stored time can be obtained from the sample 
*              : function RTC_GetTimeCanDeepStb. The flag should be set to notify 
*              : that this function has been called after returning from deep 
*              : standby mode.
*              : Also, when the IOKEEP bit in the deep standby cancel source flag 
*              : register (DSFR) is "1", the retention of the pin state can be 
*              : released by setting the IOKEEP bit to "0".
* Arguments    : none
* Return Value : none
*******************************************************************************/
void STB_CancelDeepStandby(void)
{
    volatile uint16_t dummy_buf_16b;
    volatile uint8_t  dummy_buf_8b;

    /* ==== Initialize variables ==== */
    stb_rtc_state = STB_NO_GENERATE_ALARM_INT;

    /* ==== If deep standby mode is cancelled by alarm interrupt of real time clock ==== */
    if (RZA_IO_RegRead_16(&CPG.DSFR, CPG_DSFR_RTCARF_SHIFT, CPG_DSFR_RTCARF) == 1)
    {
        /* Clear alarm flag */
        RZA_IO_RegWrite_8(&RTC.RCR1, 0, RTC_RCR1_AF_SHIFT, RTC_RCR1_AF);
        dummy_buf_8b = RTC.RCR1;    /* Dummy read */

        /* Clear cancel source flag */
        RZA_IO_RegWrite_16(&CPG.DSFR, 0, CPG_DSFR_RTCARF_SHIFT, CPG_DSFR_RTCARF);
        dummy_buf_16b = CPG.DSFR;   /* Dummy read */

        /* ---- Store time of real time clock as the time when deep standby mode is cancelled ---- */
        stb_rtc_time.second.value = RTC.RSECCNT;    /* Second counter      */
        stb_rtc_time.minute.value = RTC.RMINCNT;    /* Minute counter      */
        stb_rtc_time.hour.value   = RTC.RHRCNT;     /* Hour counter        */
        stb_rtc_time.week.value   = RTC.RWKCNT;     /* Day of week counter */
        stb_rtc_time.day.value    = RTC.RDAYCNT;    /* Day counter         */
        stb_rtc_time.month.value  = RTC.RMONCNT;    /* Month counter       */
        stb_rtc_time.year.value   = RTC.RYRCNT;     /* Year counter        */

        /* Time of deep standby mode cancellation is stored */
        stb_rtc_state = STB_GENERATE_ALARM_INT;
    }

    /* ==== If pin state of deep standby mode is retained ==== */
    if (RZA_IO_RegRead_16(&CPG.DSFR, CPG_DSFR_IOKEEP_SHIFT, CPG_DSFR_IOKEEP) == 1)
    {
        /* Release pin without default memory control pins */
        RZA_IO_RegWrite_16(&CPG.DSFR, 0, CPG_DSFR_IOKEEP_SHIFT, CPG_DSFR_IOKEEP);
        dummy_buf_16b = CPG.DSFR;   /* Dummy read */
    }
}

/*******************************************************************************
* Function Name: RTC_GetTimeCanDeepStb
* Description  : Stores the time of deep standby mode cancellation in the area 
*              : specified by the argument time.
*              : The sample function STB_CancelDeepStandby is used to store the 
*              : time when the deep standby mode is cancelled. Therefore, this 
*              : function should be called after the function STB_CancelDeepStandby 
*              : is executed.
*              : If this function return STB_NO_GENERATE_ALARM_INT, time of deep 
*              : standby mode cancellation is not stored to the argument time.
* Arguments    : rtc_time_t * time : Time of deep standby mode cancellation
*              :                   :   time->second.value  : Second      (0 to 59)
*              :                   :   time->minute.value  : Minute      (0 to 59)
*              :                   :   time->hour.value    : Hour        (0 to 23)
*              :                   :   time->week.value    : Day of week (0 to 6)
*              :                   :   time->day.value     : Day         (1 to 31)
*              :                   :   time->month.value   : Month       (1 to 12)
*              :                   :   time->year.value    : Year        (0 to 9999)
* Return Value : STB_GENERATE_ALARM_INT    : The state where deep standby mode 
*              :                           : was canceled by RTC alarm interrupt
*              : STB_NO_GENERATE_ALARM_INT : The state where deep standby mode 
*              :                           : was not canceled by RTC alarm interrupt 
*              :                           : or the state started from power-on 
*              :                           : reset
*******************************************************************************/
uint32_t RTC_GetTimeCanDeepStb(rtc_time_t * time)
{
    time->second.value = stb_rtc_time.second.value;
    time->minute.value = stb_rtc_time.minute.value;
    time->hour.value   = stb_rtc_time.hour.value;
    time->week.value   = stb_rtc_time.week.value;
    time->day.value    = stb_rtc_time.day.value;
    time->month.value  = stb_rtc_time.month.value;
    time->year.value   = stb_rtc_time.year.value;

    return stb_rtc_state;
}


/* End of File */

