/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : sample_rtc_command.c
* $Rev: 1153 $
* $Date:: 2014-08-22 13:45:42 +0900#$
* Description : Realtime Clock(RTC) sample program
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"
#include "dev_drv.h"                /* Device Driver common header */
#include "devdrv_rtc.h"             /* RTC Driver Header */
#include "command.h"
#include "sample_rtc.h"
#include "iodefine.h"
#include "rza_io_regrw.h"

/******************************************************************************
Typedef definitions
******************************************************************************/
/* ==== Itmes to be input ==== */
typedef enum rtc_time_item
{
    RTC_ITEM_WEEK,      /* Day of week */
    RTC_ITEM_MONTH,     /* Month       */
    RTC_ITEM_DAY,       /* Day         */
    RTC_ITEM_YEAR,      /* Year        */
    RTC_ITEM_HOUR,      /* Hour        */
    RTC_ITEM_MINUTE,    /* Minute      */
    RTC_ITEM_SECOND,    /* Second      */
    RTC_ITEM_TOTAL
} rtc_time_item_t;

/******************************************************************************
Macro definitions
******************************************************************************/
/* ==== Definitions to specify array of minimum value and maximum value of time ==== */
#define RTC_INPUT_RANGE_MIN     (0)     /* Definition to specify array of minimum value of effective range */
#define RTC_INPUT_RANGE_MAX     (1)     /* Definition to specify array of maximum value of effective range */

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/


/******************************************************************************
Private global variables and functions
******************************************************************************/
/* ==== Prototype declaration ==== */
static void     RTC_InputTime(rtc_time_t * time, uint32_t start_item, uint32_t end_item);
static void     RTC_InputTimeDisp(uint32_t time_item);
static int32_t  RTC_InputTimeAccept(int32_t * input_num);
static int32_t  RTC_InputTimeChkRange(uint32_t time_item, int32_t input_num);
static void     RTC_InputTimeStore(uint32_t time_item, rtc_time_t * time, int32_t input_num, uint8_t enable);
static void     RTC_InputTimeError(void);
static void     RTC_DisplayCurrentTime(void);
static void     RTC_DisplayTime(rtc_time_t * time);
static int32_t  RTC_ConvStrToNum(char_t * str, int32_t * num);

/* ==== Global variables ==== */
/* ---- Terminal output string table ---- */
static const char_t rtc_display_month[12][5] = 
{ "Jan.", "Feb.", "Mar.", "Apr.", "May.", "Jun.", "Jul.", "Aug.", "Sep.", "Oct.", "Nov.", "Dec." };
static const char_t rtc_display_week[RTC_WK_TOTAL][5] = 
{ "Sun.", "Mon.", "Tue.", "Wed.", "Thu.", "Fri.", "Sat." };
static const char_t rtc_display_input_item[RTC_ITEM_TOTAL][60] =
{
    "Day of week (Sun=0/Mon=1/Tue=2/Wed=3/Thu=4/Fri=5/Sat=6) : ",   /* Day of week */
    "Month (1 - 12) : ",    /* Month  */
    "Day (1 - 31) : ",      /* Day    */
    "Year (0 - 9999) : ",   /* Year   */
    "Hour (0 - 23) : ",     /* Hour   */
    "Minute (0 - 59) : ",   /* Minute */
    "Second (0 - 59) : "    /* Second */
};
/* ---- Time input effective range table ---- */
static const int32_t rtc_input_range[RTC_ITEM_TOTAL][2] =
{
/*   Min, Max */
    {  0,    6},    /* Day of week */
    {  1,   12},    /* Month       */
    {  1,   31},    /* Day         */
    {  0, 9999},    /* Year        */
    {  0,   23},    /* Hour        */
    {  0,   59},    /* Minute      */
    {  0,   59}     /* Second      */
};


/******************************************************************************
* Function Name: Sample_RTC_Init
* Description  : This is a sample function to initialize RTC. Called when "1"+"Enter" 
*              : keys are input during the wait processing for character input 
*              : using the sample function Sample_RTC_Main.
*              : In the sample code, RTC is initialized by using the API function 
*              : R_RTC_Init.
* Arguments    : int32_t   argc : The number of command arguments input from 
*              :                : the terminal. Not used in this function.
*              : char_t ** argv : Pointer to the command input from the terminal.
*              :                : Not used in this function.
* Return Value : COMMAND_SUCCESS : Success of RTC sample code processing
******************************************************************************/
int32_t Sample_RTC_Init(int32_t argc, char_t ** argv)
{
    printf("\n");

    /* ==== RTC initialization ==== */
    R_RTC_Init();

    printf("RTC initialize complete.\n");
    printf("\n");

    return COMMAND_SUCCESS;
}

/******************************************************************************
* Function Name: Sample_RTC_SetTime
* Description  : This is a sample function to set the time to the RTC time counter.
*              : Called when "2"+"Enter" keys are input during the wait processing 
*              : for character input using the sample function Sample_RTC_Main.
*              : In the sample code, the time (decimal digit) input from the terminal 
*              : is set to the RTC time counter using the API function R_RTC_SetCnt. 
*              : The items for the time should be input in order day of week, 
*              : month, day, year, hour, minute, and second. Inputs the values 
*              : of the specified range. If a value outside the specified range 
*              : is input, waits until appropriate value is input. When "-1" is 
*              : input, the item should not be set to the RTC time counter.
* Arguments    : int32_t   argc : The number of command arguments input from 
*              :                : the terminal. Not used in this function.
*              : char_t ** argv : Pointer to the command input from the terminal.
*              :                : Not used in this function.
* Return Value : COMMAND_SUCCESS : Success of RTC sample code processing
******************************************************************************/
int32_t Sample_RTC_SetTime(int32_t argc, char_t ** argv)
{
    rtc_time_t time;

    printf("\n");

    /* ==== Time setting (decimal digit) ==== */
#if 1   /* Generate time by inputting from terminal */
    printf("Enter time (decimal).\n");
    printf("Enter \"-1\" to the item where you do not want to change the time.\n");
    RTC_InputTime(&time, RTC_ITEM_WEEK, RTC_ITEM_SECOND);
    printf("\n");
#else   /* Generate time by setting the following parameters */
    time.second.value  = 0;                 /* 0       */
    time.minute.value  = 15;                /* 15      */
    time.hour.value    = 10;                /* Ten     */
    time.week.value    = RTC_WK_TUESDAY;    /* Tuesday */
    time.day.value     = 1;                 /* 1st     */
    time.month.value   = 4;                 /* April   */
    time.year.value    = 2014;              /* 2014    */

    time.second.enable = RTC_ENABLE;        /* Second      */
    time.minute.enable = RTC_ENABLE;        /* Minute      */
    time.hour.enable   = RTC_ENABLE;        /* Hour        */
    time.week.enable   = RTC_ENABLE;        /* Day of week */
    time.day.enable    = RTC_ENABLE;        /* Day         */
    time.month.enable  = RTC_ENABLE;        /* Month       */
    time.year.enable   = RTC_ENABLE;        /* Year        */
#endif
    R_RTC_SetCnt(&time);

    printf("RTC time counter setting complete.\n");
    printf("\n");

    return COMMAND_SUCCESS;
}

/******************************************************************************
* Function Name: Sample_RTC_GetTime
* Description  : This is a sample function to obtain time from the RTC time counter. 
*              : Called when "3"+"Enter" keys are input during wait processing 
*              : for character input using the sample function Sample_RTC_Main.
*              : In the sample code, this function displays the time obtained from 
*              : the RTC time counter to the terminal using the API function 
*              : R_RTC_GetCnt.
* Arguments    : int32_t   argc : The number of command arguments input from 
*              :                : the terminal. Not used in this function.
*              : char_t ** argv : Pointer to the command input from the terminal.
*              :                : Not used in this function.
* Return Value : COMMAND_SUCCESS : Success of RTC sample code processing
******************************************************************************/
int32_t Sample_RTC_GetTime(int32_t argc, char_t ** argv)
{
    printf("\n");

    /* ==== Display current time to terminal ==== */
    RTC_DisplayCurrentTime();

    return COMMAND_SUCCESS;
}

/******************************************************************************
* Function Name: Sample_RTC_Start
* Description  : This is a sample function to start the RTC time count operation. 
*              : Called when "4"+"Enter" keys are input during wait processing 
*              : for character input using the sample function Sample_RTC_Main. 
*              : In the sample code, this function displays the time obtained from 
*              : the RTC time counter to the terminal and starts the RTC time 
*              : count operation using the API function R_RTC_Open. 
* Arguments    : int32_t   argc : The number of command arguments input from 
*              :                : the terminal. Not used in this function.
*              : char_t ** argv : Pointer to the command input from the terminal.
*              :                : Not used in this function.
* Return Value : COMMAND_SUCCESS : Success of RTC sample code processing
******************************************************************************/
int32_t Sample_RTC_Start(int32_t argc, char_t ** argv)
{
    printf("\n");

    /* ==== Display current time to terminal ==== */
    RTC_DisplayCurrentTime();

    /* ==== Start RTC time count operation ==== */
    R_RTC_Open();

    printf("RTC time count start.\n");
    printf("\n");

    return COMMAND_SUCCESS;
}

/******************************************************************************
* Function Name: Sample_RTC_Stop
* Description  : This is a sample function to stop the RTC time count operation. 
*              : Called when "5"+"Enter" keys are input during wait processing 
*              : for character input using the sample function Sample_RTC_Main. 
*              : In the sample code, this function stops the RTC time count operation 
*              : using the API function R_RTC_Close, and displays the time obtained 
*              : from the RTC time counter to the terminal.
* Arguments    : int32_t   argc : The number of command arguments input from 
*              :                : the terminal. Not used in this function.
*              : char_t ** argv : Pointer to the command input from the terminal.
*              :                : Not used in this function.
* Return Value : COMMAND_SUCCESS : Success of RTC sample code processing
******************************************************************************/
int32_t Sample_RTC_Stop(int32_t argc, char_t ** argv)
{
    printf("\n");

    /* ==== Stop RTC time count operation ==== */
    R_RTC_Close();

    printf("RTC time count stop.\n");
    printf("\n");

    /* ==== Display current time to terminal ==== */
    RTC_DisplayCurrentTime();

    return COMMAND_SUCCESS;
}

/******************************************************************************
* Function Name: Sample_RTC_DeepStandby
* Description  : This is a sample processing to transit to deep standby mode 
*              : after selecting alarm interrupt as deep standby mode cancel source. 
*              : Called when "6"+"Enter" keys are input during wait processing
*              : for character input using the sample function Sample_RTC_Main.
*              : In the sample code, the time values input from the terminal are 
*              : specified for the hour alarm register (RHRAR) and the minute 
*              : alarm register (RMINAR), and "0" for the second alarm register 
*              : (RSECAR) to generate alarm interrupt at the time (hour and minute) 
*              : input from the terminal. Then the ENB bits of each register 
*              : are set. Reads the time from the RTC time counter and displays 
*              : it to the terminal as the time to transit to deep standby mode.
*              : Sets the STB so that the cancel deep standby mode is cancelled 
*              : when the RTC alarm interrupt is generated, and transits to deep 
*              : standby mode. The STB settings are made to activation through 
*              : the external memory (NOR flash memory connected to the CS0 space) 
*              : after the deep standby mode is cancelled.
*              : When both the time (Hour, Minute) input from the terminal is 
*              : "-1", or when reading time from the RTC time counter and an 
*              : error occurs, COMMAND_ERROR is returned by this function.
* Arguments    : int32_t   argc : The number of command arguments input from 
*              :                : the terminal. Not used in this function.
*              : char_t ** argv : Pointer to the command input from the terminal.
*              :                : Not used in this function.
* Return Value : COMMAND_SUCCESS : Success of RTC sample code processing
*              : COMMAND_ERROR   : Failure of RTC sample code processing
******************************************************************************/
int32_t Sample_RTC_DeepStandby(int32_t argc, char_t ** argv)
{
    rtc_time_t       time;
    rtc_time_t       alarm_time;
    rtc_alarm_enb_t  alarm_enb;
    int32_t          ret;

    printf("\n");

    /* ==== Input alarm time (hour and minute) from terminal ==== */
    printf("Transit to deep standby mode.\n");
    printf("Enter time to cancel deep standby mode (decimal).\n");
    printf("Enter \"-1\" to the item where you set current time.\n");
    RTC_InputTime(&alarm_time, RTC_ITEM_HOUR, RTC_ITEM_MINUTE);
    printf("\n");

    if ((RTC_ENABLE == alarm_time.minute.enable) || (RTC_ENABLE == alarm_time.hour.enable))
    {
        /* ==== Obtain time ==== */
         time.second.enable = RTC_ENABLE;    /* Second      */
         time.minute.enable = RTC_ENABLE;    /* Minute      */
         time.hour.enable   = RTC_ENABLE;    /* Hour        */
         time.week.enable   = RTC_ENABLE;    /* Day of week */
         time.day.enable    = RTC_ENABLE;    /* Day         */
         time.month.enable  = RTC_ENABLE;    /* Month       */
         time.year.enable   = RTC_ENABLE;    /* Year        */
        if (R_RTC_GetCnt(&time) == DEVDRV_SUCCESS)
        {
            /* ==== Display time for transition to deep standby mode to terminal ==== */
            printf("Transit to deep standby mode at..\n");
            RTC_DisplayTime(&time);

            /* ==== Alarm setting ==== */
            /* ---- Set the time input from terminal to time and minute, set "0" to second ---- */
            time.second.value = 0;
            if (RTC_ENABLE == alarm_time.minute.enable)
            {
                time.minute.value = alarm_time.minute.value;
            }
            if (RTC_ENABLE == alarm_time.hour.enable)
            {
                time.hour.value   = alarm_time.hour.value;
            }
            /* ==== Alarm-specified information (specifies hour, minute, and second as an alarm target) ==== */
            alarm_enb.second = 1;   /* Second alarm validated        */
            alarm_enb.minute = 1;   /* Minute alarm validated        */
            alarm_enb.hour   = 1;   /* Hour alarm validated          */
            alarm_enb.week   = 0;   /* Day of week alarm invalidated */
            alarm_enb.day    = 0;   /* Day alarm invalidated         */
            alarm_enb.month  = 0;   /* Month alarm invalidated       */
            alarm_enb.year   = 0;   /* Year alarm invalidated        */
            R_RTC_SetAlarm(&time, &alarm_enb);

            /* ==== Display estimated time for deep standby mode cancellation to terminal ==== */
            printf("Deep standby mode will be canceled at %02d:%02d:00.\n", 
                                          time.hour.value, time.minute.value);
            printf("\n");
            printf("\n");

            /* After RTC register setting immediately before the transition to deep standby mode, */
            /* the dummy read of the register is necessary.                                       */
            R_RTC_GetAlarm(&time, &alarm_enb);  /* Dummy read */

            /* ==== Transit to deep standby mode ==== */
            Sample_RTC_TransitionDeepStandby();

            ret = COMMAND_SUCCESS;
        }
        else
        {
            ret = COMMAND_ERROR;
        }
    }
    else
    {
        ret = COMMAND_ERROR;
    }

    return ret;
}

/******************************************************************************
* Function Name: RTC_InputTime
* Description  : Stores the time (decimal digit) input from the terminal in the 
*              : argument time. The items for the time should be input in order 
*              : day of week, month, day, year, hour, minute, and second. This 
*              : function waits until the values of the specified range are input. 
*              : If a value outside the specified range is input, waits until 
*              : an appropriate value is input. When the value of the range is 
*              : input, stores the input value and RTC_ENABLE in the member value 
*              : and the member enable of the argument time respectively.
*              : If "-1" is input, RTC_DISABLE and "0" should be stored in the 
*              : member enable and the member value respectively. 
*              : Continues input from the items specified by the argument start_item 
*              : to the items specified by the argument end_item.
* Arguments    : rtc_time_t * time   : Time storage area
*              : uint32_t start_item : Items to start input
*              :                     :   RTC_ITEM_WEEK   : Day of week
*              :                     :   RTC_ITEM_MONTH  : Month
*              :                     :   RTC_ITEM_DAY    : Day
*              :                     :   RTC_ITEM_YEAR   : Year
*              :                     :   RTC_ITEM_HOUR   : Hour
*              :                     :   RTC_ITEM_MINUTE : Minute
*              :                     :   RTC_ITEM_SECOND : Second
*              : uint32_t end_item   : Items to stop input
*              :                     :   RTC_ITEM_WEEK   : Day of week
*              :                     :   RTC_ITEM_MONTH  : Month
*              :                     :   RTC_ITEM_DAY    : Day
*              :                     :   RTC_ITEM_YEAR   : Year
*              :                     :   RTC_ITEM_HOUR   : Hour
*              :                     :   RTC_ITEM_MINUTE : Minute
*              :                     :   RTC_ITEM_SECOND : Second
* Return Value : none
******************************************************************************/
static void RTC_InputTime(rtc_time_t * time, uint32_t start_item, uint32_t end_item)
{
    int32_t  input_num;
    uint32_t time_item;
    int32_t  ret;

    time_item = start_item;

    printf("\n");

    /* ==== Accept time input in order day of week, month, day, year, hour, minute, and second ==== */
    while (time_item <= end_item)
    {
        /* ==== Time input ==== */
        RTC_InputTimeDisp(time_item);
        ret = RTC_InputTimeAccept(&input_num);

        /* ---- When values are input ---- */
        if (0 == ret)
        {
            /* ==== Check the range of input items ==== */
            ret = RTC_InputTimeChkRange(time_item, input_num);

            if ((0 == ret) || ((-1) == ret))
            {
                /* ---- When the values are in effective range ---- */
                if (0 == ret)
                {
                    /* Store input values in member value of the argument */
                    /* time and RTC_ENABLE in member enable               */
                    RTC_InputTimeStore(time_item, time, input_num, RTC_ENABLE);
                }
                /* ---- When the value "-1" is input ---- */
                else
                {
                    /* Store a dummy value ("0") in member value of the */
                    /* argument time and RTC_DISABLE in member enable   */
                    RTC_InputTimeStore(time_item, time, 0, RTC_DISABLE);
                }

                time_item++;    /* Input next item */
            }
            /* ---- When the values are outside the effective range ---- */
            else
            {
                RTC_InputTimeError();
            }
        }
        /* ---- When something other than values is input ---- */
        else
        {
            RTC_InputTimeError();
        }
    }
}

/******************************************************************************
* Function Name: RTC_InputTimeDisp
* Description  : Display a message indicating that the time for the items specified 
*              : by the argument time_item should be input to the terminal to
*              : the terminal.
* Arguments    : uint32_t time_item : Items
*              :                    :   RTC_ITEM_WEEK   : Day of week
*              :                    :   RTC_ITEM_MONTH  : Month
*              :                    :   RTC_ITEM_DAY    : Day
*              :                    :   RTC_ITEM_YEAR   : Year
*              :                    :   RTC_ITEM_HOUR   : Hour
*              :                    :   RTC_ITEM_MINUTE : Minute
*              :                    :   RTC_ITEM_SECOND : Second
* Return Value : none
******************************************************************************/
static void RTC_InputTimeDisp(uint32_t time_item)
{
    printf("%s", rtc_display_input_item[time_item]);
}

/******************************************************************************
* Function Name: RTC_InputTimeAccept
* Description  : Waits for character input from the terminal. Converts the input 
*              : character string into values, and stores them in the argument 
*              : input_num.
* Arguments    : int32_t * input_num : Storage area for input values
* Return Value :  0 : Input character string indicates values
*              : -1 : Input character string indicates other than values
******************************************************************************/
static int32_t RTC_InputTimeAccept(int32_t * input_num)
{
    char_t  input_str[256];
    int32_t ret;

    /* ==== Wait for character input ==== */
    if (gets(input_str) == NULL)
    {
        printf("error: gets error\n");
        while (1)
        {
            /* Do Nothing */
        }
    }

    /* ==== Convert character string into values ==== */
    ret = RTC_ConvStrToNum(input_str, input_num);

    return ret;
}

/******************************************************************************
* Function Name: RTC_InputTimeChkRange
* Description  : Determines whether the value specified by the argument input_num 
*              : is in or outside the effective range, or indicates "-1", and 
*              : returns the result. The effective range for the items specified 
*              : by the argument time_item is used for judgment.
* Arguments    : uint32_t time_item : Items
*              :                    :   RTC_ITEM_WEEK   : Day of week
*              :                    :   RTC_ITEM_MONTH  : Month
*              :                    :   RTC_ITEM_DAY    : Day
*              :                    :   RTC_ITEM_YEAR   : Year
*              :                    :   RTC_ITEM_HOUR   : Hour
*              :                    :   RTC_ITEM_MINUTE : Minute
*              :                    :   RTC_ITEM_SECOND : Second
*              : int32_t input_num  : Input values
* Return Value :  0 : Input value is in effective range
*              : -1 : Input value is "-1"
*              : -2 : Input value is outside the effective range
******************************************************************************/
static int32_t RTC_InputTimeChkRange(uint32_t time_item, int32_t input_num)
{
    int32_t ret;

    /* ==== When the input values are in effective range ==== */
    if ((input_num >= rtc_input_range[time_item][RTC_INPUT_RANGE_MIN]) &&
        (input_num <= rtc_input_range[time_item][RTC_INPUT_RANGE_MAX]))
    {
        ret = 0;
    }
    /* ==== When the input value is "-1" ==== */
    else if ((-1) == input_num)
    {
        ret = -1;
    }
    /* ==== When the input values are outside the effective range ==== */
    else
    {
        ret = -2;
    }

    return ret;
}

/******************************************************************************
* Function Name: RTC_InputTimeStore
* Description  : Generates the time structured variable (time) used as an argument 
*              : of the function R_RTC_SetCnt. Regarding the items specified by 
*              : the argument time_item, the values specified by the arguments 
*              : input_num and enable are stored in the member value and the 
*              : member enable of the argument time respectively. 
* Arguments    : uint32_t time_item : Items
*              :                    :   RTC_ITEM_WEEK   : Day of week
*              :                    :   RTC_ITEM_MONTH  : Month
*              :                    :   RTC_ITEM_DAY    : Day
*              :                    :   RTC_ITEM_YEAR   : Year
*              :                    :   RTC_ITEM_HOUR   : Hour
*              :                    :   RTC_ITEM_MINUTE : Minute
*              :                    :   RTC_ITEM_SECOND : Second
*              : rtc_time_t * time  : Storage area for time
*              : int32_t input_num  : Value to be set to member value of the 
*              :                    : argument time (Values input)
*              : uint8_t enable     : Value to be set to member enable of the 
*              :                    : argument time (Specifying operation to 
*              :                    : the function R_RTC_SetCnt)
*              :                    :   RTC_ENABLE  : Specification to set the time
*              :                    :   RTC_DISABLE : Specification not to set the time
* Return Value : none
******************************************************************************/
static void RTC_InputTimeStore(uint32_t time_item, rtc_time_t * time, int32_t input_num, uint8_t enable)
{
    /* ==== Store in time structure ==== */
    switch (time_item)
    {
        /* ---- Day of week ---- */
        case RTC_ITEM_WEEK:
            time->week.value  = (uint8_t)input_num;
            time->week.enable = enable;
        break;
        /* ---- Month ---- */
        case RTC_ITEM_MONTH:
            time->month.value  = (uint8_t)input_num;
            time->month.enable = enable;
        break;
        /* ---- Day ---- */
        case RTC_ITEM_DAY:
            time->day.value  = (uint8_t)input_num;
            time->day.enable = enable;
        break;
        /* ---- Year ---- */
        case RTC_ITEM_YEAR:
            time->year.value  = (uint16_t)input_num;
            time->year.enable = enable;
        break;
        /* ---- Hour ---- */
        case RTC_ITEM_HOUR:
            time->hour.value  = (uint8_t)input_num;
            time->hour.enable = enable;
        break;
        /* ---- Minute ---- */
        case RTC_ITEM_MINUTE:
            time->minute.value  = (uint8_t)input_num;
            time->minute.enable = enable;
        break;
        /* ---- Second ---- */
        case RTC_ITEM_SECOND:
            time->second.value  = (uint8_t)input_num;
            time->second.enable = enable;
        break;
        default:
            /* Do not reach here based on the assumption */
        break;
    }
}

/******************************************************************************
* Function Name: RTC_InputTimeError
* Description  : Displays a message indicating that the input values are invalid
*              : to the terminal.
* Arguments    : none
* Return Value : none
******************************************************************************/
static void RTC_InputTimeError(void)
{
    printf("error: input error\n");
    printf("\n");
}

/******************************************************************************
* Function Name: RTC_DisplayCurrentTime
* Description  : Obtains time using the API function R_RTC_GetCnt and displays 
*              : it to the terminal.
* Arguments    : none
* Return Value : none
******************************************************************************/
static void RTC_DisplayCurrentTime(void)
{
    rtc_time_t time;
    int32_t    ret;

    /* ==== Obtain time ==== */
    time.second.enable = RTC_ENABLE;    /* Second      */
    time.minute.enable = RTC_ENABLE;    /* Minutes     */
    time.hour.enable   = RTC_ENABLE;    /* Hour        */
    time.week.enable   = RTC_ENABLE;    /* Day of week */
    time.day.enable    = RTC_ENABLE;    /* Day         */
    time.month.enable  = RTC_ENABLE;    /* Month       */
    time.year.enable   = RTC_ENABLE;    /* Year        */
    ret = R_RTC_GetCnt(&time);

    if (DEVDRV_SUCCESS == ret)
    {
        RTC_DisplayTime(&time);
    }
    else
    {
        printf("error: Failure in R_RTC_GetCnt\n");
        printf("\n");
    }
}

/******************************************************************************
* Function Name: RTC_DisplayTime
* Description  : Displays the time (decimal digit) specified by the argument 
*              : time to the terminal.
* Arguments    : rtc_time_t * time : Time
* Return Value : none
******************************************************************************/
static void RTC_DisplayTime(rtc_time_t * time)
{
    printf("------------------------------\n");
    printf("%4s %4s %2d, %4d at %02d:%02d:%02d\n", 
            rtc_display_week[time->week.value], 
            rtc_display_month[time->month.value - 1], 
            time->day.value, 
            time->year.value, 
            time->hour.value, 
            time->minute.value, 
            time->second.value);
    printf("------------------------------\n");
    printf("\n");
}

/******************************************************************************
* Function Name: RTC_ConvStrToNum
* Description  : Converts the character string specified by the argument str into 
*              : numerical data and stores it in the argument num.
*              : Converts the series of numbers ("0" to "9") placed at the start 
*              : of character string into numerical values. However, the series 
*              : of space character placed at the start of character string should 
*              : be ignored. When the start of character string has "+" or "-", 
*              : these are used as symbols for numerical values. If the start of 
*              : character string does not have numbers ("0" to "9"), conversion 
*              : should not be performed, instead, "-1" is returned.
* Arguments    : char_t * str  : Character string
*              : int32_t * num : Storage area for values
* Return Value :  0 : Conversion succeeded
*              : -1 : Conversion failed (Start of character string indicates 
*              :    : other than numbers)
******************************************************************************/
static int32_t RTC_ConvStrToNum(char_t * str, int32_t * num)
{
    uint32_t cnt;
    int32_t  sing;
    int32_t  ret;

    *num = 0;
    ret = -1;

    /* ==== Ignore the series of space characters placed at the start of character string ==== */
    for (cnt = 0; str[cnt] == ' '; cnt++)
    {
        /* Do Nothing */
    }

    /* ==== Use "+" and "-" as symbols ==== */
    if (str[cnt] == '-')
    {
        sing = -1;
        cnt++;
    }
    else
    {
        sing = 1;
        if (str[cnt] == '+')
        {
            cnt++;
        }
    }

    /* ==== Convert the series of numbers into numerical values ==== */
    for (; (str[cnt] >= '0') && (str[cnt] <= '9'); cnt++)
    {
        *num = (10 * (*num)) + (str[cnt] - '0');
        ret = 0;
    }

    /* ==== Reflect symbols ==== */
    if (0 == ret)
    {
        *num = (*num) * sing;
    }

    return ret;
}

/* End of File */

