/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : sample_rtc_deep_standby.c
* $Rev: 1155 $
* $Date:: 2014-08-26 15:37:03 +0900#$
* Description : Realtime Clock(RTC) sample program
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"
#include "sample_rtc.h"
#include "iodefine.h"
#include "rza_io_regrw.h"

/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/


/******************************************************************************
Private global variables and functions
******************************************************************************/
static void RTC_SetPinStatus(void);


/******************************************************************************
* Function Name: Sample_RTC_TransitionDeepStandby
* Description  : This is a sample processing to transit to the deep standby mode.
*              : In the sample code, alarm interrupt is selected as a factor for 
*              : cancellation of deep standby mode to transit to the deep standby 
*              : mode.
*              : After the deep standby mode is cancelled, STB settings are made 
*              : to activation through the external memory (NOR flash memory 
*              : connected to CS0 space).
* Arguments    : none
* Return Value : none
******************************************************************************/
void Sample_RTC_TransitionDeepStandby(void)
{
    volatile uint32_t dummy_buf_32b;
    volatile uint16_t dummy_buf_16b;
    volatile uint8_t  dummy_buf_8b;

    /* IRQ interrupt and FIQ interrupt disabled */
    __disable_irq();
    __disable_fiq();

    /* ---- standby_mode_en bit of Power Control Register setting ---- */
    L2C.REG15_POWER_CTRL = 0x00000001uL;
    dummy_buf_32b = L2C.REG15_POWER_CTRL;   /* Dummy read */

    /* ---- Activation through the external memory after deep standby mode is cancelled ---- */
    CPG.DSCTR = 0x00u;              /* EBUSKEEPE = 0, RAMBOOT = 0 */
    dummy_buf_8b = CPG.DSCTR;       /* Dummy read                 */

    /* ---- Deep standby mode is cancelled by RTC alarm interrupt ---- */
    RZA_IO_RegWrite_16(&CPG.DSSSR, 1, CPG_DSSSR_RTCAR_SHIFT, CPG_DSSSR_RTCAR);
    dummy_buf_16b = CPG.DSSSR;          /* Dummy read            */

    /* ---- Settings for pin state ---- */
    RTC_SetPinStatus();

    /* ---- Transit to deep standby mode by WFI instruction ---- */
    CPG.STBCR1 = 0xC0u;             /* STBY = 1, DEEP = 1 */
    dummy_buf_8b = CPG.STBCR1;      /* Dummy read         */

    /* ---- Clear deep standby mode cancel source flag ---- */
    if (RZA_IO_RegRead_16(&CPG.DSFR, CPG_DSFR_RTCARF_SHIFT, CPG_DSFR_RTCARF) == 1)
    {
        /* Clear RTCAR flag */
        RZA_IO_RegWrite_16(&CPG.DSFR, 0, CPG_DSFR_RTCARF_SHIFT, CPG_DSFR_RTCARF);
        dummy_buf_16b = CPG.DSFR;   /* Dummy read */
    }

    /* ---- Settings not to notify interrupts except NMI to CPU ---- */
    INTC.ICCICR = 0x00000000uL;     /* CPU Interface Control Register = 0 */
    dummy_buf_32b = INTC.ICCICR;    /* Dummy read                         */

    while (1)
    {
        /* ---- WFI instruction ---- */
        __wfi();
    }
}

/******************************************************************************
* Function Name: RTC_SetPinStatus
* Description  : Sets pin state in deep standby mode.
* Arguments    : none
* Return Value : none
******************************************************************************/
static void RTC_SetPinStatus(void)
{
    volatile uint16_t dummy_buf_16b;
    volatile uint8_t  dummy_buf_8b;

    /* ==== Settings for pin state ==== */
    /* NOTE: In the sample code, it is assumed that the power of NOR flash memory */
    /*       should not be shut off during transition to deep standby mode.       */
    /*       Therefore, the output pins are set to keep the output state, not     */
    /*       impedance state.                                                     */

    /* Set CKIO pin to low-level output in deep standby mode */
    RZA_IO_RegWrite_16(&CPG.FRQCR, 1, CPG_FRQCR_CKOEN_SHIFT, CPG_FRQCR_CKOEN);
    dummy_buf_16b = CPG.FRQCR;      /* Dummy read */

    /* Set to keep states of output pins in deep standby mode */
    RZA_IO_RegWrite_8(&CPG.STBCR2, 0, CPG_STBCR2_HIZ_SHIFT, CPG_STBCR2_HIZ);
    dummy_buf_8b = CPG.STBCR2;      /* Dummy read */

    /* Set A25 to A0, BS#, CSn#, RD/WR#, WEn#/DQMxx/AH#, and RD# pins to drive  */
    /* states in deep standby mode                                              */
    RZA_IO_RegWrite_32(&BSC.CMNCR, 1, BSC_CMNCR_HIZMEM_SHIFT, BSC_CMNCR_HIZMEM);

    /* Set CKE, RAS#, and CAS# pins to drive states in deep standby mode */
    RZA_IO_RegWrite_32(&BSC.CMNCR, 1, BSC_CMNCR_HIZCNT_SHIFT, BSC_CMNCR_HIZCNT);
}


/* End of File */

